## ========================================================================================
## MCI Electronics www.mcielectronics.cl  
## 
## Archivo:  gateway.py
## 
## Descripción: Recibe un dato a través de un LoRaBee y lo reenvia a Ubidots por un 3GBee
## 
## Dispositivo: Raspberry Pi 3/4    - MCI06110
##              3GBee Telit         - MCI01833
##              LoRaBee             - MCI05818 
## 
## ========================================================================================
##  Ver | dd mmm yyyy | Autor                 | Descripción
## =====|=============|=========|==========================================================
## 1.00 | 25 Nov 2021 | Fernando Velásquez    | Versión inicial
## ======================================================================================== 



from time import sleep

import RPi.GPIO as GPIO
import serial
import requests
import time
import json


GPIO.setwarnings(False)
GPIO.setmode(GPIO.BCM)

GPIO.setup(27, GPIO.OUT)    #Pin de encendido del 3G Bee
GPIO.setup(23, GPIO.OUT)    #Selector de socket XBee 1 para Celular 0 para Lora
GPIO.setup(24, GPIO.OUT)
GPIO.output(24, False)
lora = serial.Serial("/dev/ttyS0", 57600)
celular = serial.Serial("/dev/ttyS0", 9600)
celular.close()
lora.close()
APIKEY = "EA274A8ADBDF13DAF2D396F8F517A"                 #APIKEY del dispositivo en Ubidots
GPIO.output(27, False)
sleep(2)


#========================Funciones para el funcionamiento del LoRaBee==================================
#Esta función se encarga de hacer todas las configuraciones iniciales del modulo LoRaBee
def inicializa_lora():
    print("Inicializando modulo LoRa...")
    send_lora(b"sys get ver\r\n")
    send_lora(b"mac pause\r\n")
    send_lora(b"radio set mod lora\r\n")
    send_lora(b"radio set freq 923300000\r\n")
    send_lora(b"radio set pwr 14\r\n")
    send_lora(b"radio set sf sf7\r\n")
    send_lora(b"radio set afcbw 41.7\r\n")
    send_lora(b"radio set rxbw 125\r\n")
    send_lora(b"radio set prlen 8\r\n")
    send_lora(b"radio set crc on\r\n")
    send_lora(b"radio set iqi off\r\n")
    send_lora(b"radio set cr 4/5\r\n")
    send_lora(b"radio set sync 12\r\n")
    send_lora(b"radio set bw 125\r\n")
    send_lora(b"radio set pwr 14\r\n")
    print("Modulo LoRa Inicializado")

#Esta funcion es la encargada de enviar los comandos al modulo LoRaBee
def send_lora(mensaje):
    GPIO.output(23, False)  #Selecciona el multiplexor para LoRa y abre la comunicación serial.
    if not lora.is_open:
        lora.open()
    lora.reset_input_buffer()
    lora.reset_output_buffer()
    lora.write(mensaje)
    sleep(0.1)
    if lora.in_waiting:
        respuesta = lora.readline()
    else:
        respuesta = "[ERROR] No hubo respuesta del modulo LoRa"
    print(respuesta)       #Imprime la respuesta del modulo LoRaBee, descomenta para DEBUG
    lora.close()
    return respuesta

#========================Funciones para el funcionamiento del 3GBee==================================
def ModemON():
    rCode = 0
    GPIO.output(27, True)      #Selecciona el multiplexor para 3GBee
    if send_3G(b'AT\r', "OK"):      
        print("[INFO] Modem ON")
        rCode = 1
    return rCode


#Esta funcion se encarga de verificar si hay SIMCARD y registrarla en la RED celular
def ConexionRED():
    rCode = 0
    estado = 1
    while rCode == 0:
        if estado == 1:
            if send_3G(b'AT+CPIN?\r', "+CPIN: READY") == 1:
                print("[INFO] SIMCARD detectada")
                estado = 2
                continue
            else:
                print("[ERROR] SIMCARD no detectada")
                return rCode
        if estado == 2: 
            if send_3G(b'AT+CREG=0\r', "OK") == 1:
                print("[INFO] Equipo registrado en la RED")
                estado = 3
                continue
            else:
                print("[ERROR] No se pudo activar la RED")
                sleep(1)
                continue
        if estado == 3:
            if send_3G(b'AT+CREG?\r', "+CREG: 0,1") == 1:
                print("[INFO] Equipo conectado a la RED celular")
                rCode = 1
                return rCode
            else:
                print("[ERROR] No se puede conectar a la RED celular")
                continue


#Esta es la funcion que se encarga de configurar el servidor a donde se enviaran los datos
def ConectarServidor(intentos):
    rCode = 0
    for i in range(intentos):
        if send_3G(b'AT#SD=1,0,9012,\"industrial.api.ubidots.com\",0,0,0\r', "CONNECT", 4): #En este comando configuras la IP y Puerto a donde vas a transmitir los datos
            rCode = 1
            break    
        else:
            print("[ERROR] No pude conectarme al Ubidots")
            send_3G(b'+++', "OK", 4)
    return rCode

#Esta es la funcion que se encarga de configurar el APN de la SIMCARD
def ConfigAPN():
    rCode = 0
    estado = 1
    while rCode == 0:
        if estado == 1:   
            if send_3G(b'AT+CGDCONT=1,\"IP\",\"m2m.movistar.cl\"\r', "OK") == 1:        #En este comando configuras el APN de tu SIMCARD, si quieres contratar SIMCARD para IOT
                print("[INFO] APN configurado")                                         #escribenos a: adelafuente@mcitelecom.com
                estado = 2
                continue
            else:
                print("[ERROR] No se pudo configurar el APN")
                return rCode
        if estado == 2:
            if send_3G(b'AT#SGACT?\r', "#SGACT: 1,0"):
                estado = 3
                continue
            else:
                print('[INFO] Contexto activado')
                estado = 4
                continue
        if estado == 3:
            if send_3G(b'AT#SGACT=1,1\r', "#SGACT:", 8) == 1:
                print("[INFO] Contexto activado")
                estado = 4
                continue
            else:
                print("[ERROR] No se pudo activar el contexto")
        if estado == 4:
            if send_3G(b'AT&K0\r', "OK") == 1:
                print("[INFO] Desactivado el control de flujo")
                rCode = 1
            else:
                print("[ERROR] No se pudo desactivar el control de flujo")
                continue
    return rCode

def inicializando_celular():
    print("Inicializando modulo 3GBee")
    rCode = 0
    estado = 1
    while (rCode) == 0:
        if estado == 1:
            if ModemON() == 1:
                estado = 2              
                continue
            else:
                estado = 1
                continue
        if estado == 2:
            if ConexionRED() == 1:
                estado = 3
            else:
                estado = 2
                continue
        if estado == 3:
            if ConfigAPN() == 1:
                rCode = 1
                continue
            else:
                estado = 3
    return rCode

#Esta es la funcion que se encarga de enviar los datos al servidor configurado
def EnviaDatos(mensaje, respuesta, timeout_server = 4):
    GPIO.output(23,True)
    rCode = 0
    res_server = ""
    celular.apply_settings({'timeout' : timeout_server})
    if not celular.is_open:
        celular.open()
    celular.write(mensaje)
    #print("[DEBUG] Comando enviado: {}".format(mensaje))            #Muestra el comando que se esta enviando, descomenta para debuguear
    sleep(timeout_server)
    res_server = celular.readall().decode("ASCII")
    #print("[DEBUG] Respuesta del servidor {}" .format(res_server))  #Muestra la respuesta del servidor, descomenta para debuguear
    if res_server == respuesta:
        print("[INFO] Datos enviados de manera exitosa")
        send_3G(b'+++', "OK", 4)
        rCode = 1
    else:
        print("[ERROR] No se pudo enviar los datos al servidor")
        print("[ERROR] Respuesta del servidor {}" .format(res_server))
    return rCode

#Esta es la funcion que se encarga de enviar comando al modulo 3GBee
def send_3G(mensaje, respuesta, timeout_comando = 0.5):
    GPIO.output(23, True)
    rCode = 0
    res_decode = ""
    celular.apply_settings({'timeout' : timeout_comando})
    if not celular.is_open:
        celular.open()
    celular.reset_input_buffer()
    celular.reset_output_buffer()
    celular.write(mensaje)
    #print("[DEBUG] Comando enviado: {}".format(mensaje))    #Muestra el comando que se esta enviando, descomenta para debuguear
    while not celular.in_waiting:
        pass
    if celular.in_waiting:
        respuestas_modem = celular.readlines()
        for res in respuestas_modem:
            if res.decode("ASCII").rstrip().find(respuesta) != -1:
                #res_decode = res.decode("ASCII").rstrip()   #Almacena la respuesta del modem, debes descomenarla si quieres debugear la respuesta del Modulo 3GBee
                rCode = 1
    else:
        print("[ERROR] No hubo respuesta del modulo celular")
    celular.close()
    #print('[DEBUG] {}' .format(res_decode))                 #Muestra la respuesta del Modulo 3GBee, descomenta para debuguear
    return rCode

#----------------Estructura principal del programa-------------

def main():
    inicializa_lora()
    inicializando_celular()
    
    
    while True:
        print("Esperando por mensaje LoRa")
        send_lora(b"radio rx 0\r\n")
        respuesta = ""
        GPIO.output(23, False)
        lora.open()
        while respuesta == "":                          #Se queda en un bucle esperando un mensaje entrante por el Modulo LoRaBee,
            if lora.in_waiting:                         #Si se alcanza el timeout recibe un error y se sale del bucle
                respuesta = lora.readline()             
        lora.close()                                
        if respuesta == b'radio_err\r\n':
            print("[INFO] Se agotó el tiempo de espera y no se recibieron datos")
            continue
        temperatura_nodo = int(respuesta[10:-2].decode("ASCII"), 16)
        print("El dato recibido por la red LoRa es: {}" .format(temperatura_nodo))
        print("Enviado el dato al servidor configurado")
        if ConectarServidor(4) == 1:
            paquete = 'GATEWAY-HAT|POST|'+APIKEY+'|Gateway-Hat=>Temperatura:'+str(temperatura_nodo)+'|end' #Remplaza "API-KEY DEL DISPOSITIVO" con la APIKEY correspondiente de tu dispositivo en Ubidots
            EnviaDatos(paquete.encode("utf-8"), "OK")

if __name__ == "__main__":
    try:
        main()
    except KeyboardInterrupt:
        print('****** Fin de ejecucion ******')
